VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "ImageMapItem"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
'\\ --[ ImageMapItem ]----------------------------------------------------------
'\\ Represents a single clickable region on an image map
'\\
'\\ ----------------------------------------------------------------------------
'\\ (c) 2001 Merrion Computing Ltd http://www.merrioncomputing.com
'\\ ----------------------------------------------------------------------------

Option Explicit

'\\ Enumerated types for the public interface to this ImageMapItem class
Public Enum enImageMapItemShapes
   imsRectangle = 0 '"rect" or "rectangle"
   imsCircle = 1    '"circ" or "circle"
   imsPolygon = 2   '"poly" or "polygon"
End Enum

Public Enum enImageMapTargets
    imt_none = 0 '""
    imt_blank = 1 '"_blank"
    imt_parent = 2 '"_parent"
    imt_search = 3 '"_search"
    imt_self = 4 '"_self"
    imt_top = 5  '"_top"
End Enum

'\\ Constants for formatting the <area> tag...
Private Const HTML_AREATAG = "<area"
Private Const HTML_ID = "id"
Private Const HTML_SHAPE = "shape"
Private Const HTML_COORDINATES = "coords"
Private Const HTML_HREF = "href"
Private Const HTML_TARGET = "target"
Private Const HTML_AREATAGCLOSE = ">"

'\\ Private member variables
Private mShape As enImageMapItemShapes
Private mNoHref As Boolean ' "True" or "False"
Private mHref As String
Private mCoords As String
Private msId As String
Private msTarget As enImageMapTargets

'\\ Region related functions
Private mRegion As Long

'\\ Region related API calls
Private Type POINTAPI
    x As Long
    y As Long
End Type
Private Declare Function CreateRectRgnApi Lib "gdi32" Alias "CreateRectRgn" (ByVal X1 As Long, ByVal Y1 As Long, ByVal X2 As Long, ByVal Y2 As Long) As Long
Private Declare Function CreatePolygonRgnApi Lib "gdi32" Alias "CreatePolygonRgn" (lpPoint As POINTAPI, ByVal nCount As Long, ByVal nPolyFillMode As Long) As Long
Private Declare Function CreateEllipticRgnApi Lib "gdi32" Alias "CreateEllipticRgn" (ByVal X1 As Long, ByVal Y1 As Long, ByVal X2 As Long, ByVal Y2 As Long) As Long
Private Declare Function PtInRegionApi Lib "gdi32" Alias "PtInRegion" (ByVal hRgn As Long, ByVal xPos As Long, ByVal yPos As Long) As Long

Private mHrgn As Long

'\\--[ContainsPoint]-----------------------------------------------------------
'\\ Returns true if the region that this ImageMapItem represents contains
'\\ the point represented by the x and y position passed in
'\\ ---------------------------------------------------------------------------
Public Function ContainsPoint(ByVal xPos As Long, yPos As Long) As Boolean

If Me.RegionHandle = 0 Then
    ContainsPoint = False
Else
    ContainsPoint = PtInRegionApi(mHrgn, xPos, yPos)
End If

End Function




Friend Property Get CoordinatePoints() As Collection

Dim ptThis As APIPoint
Dim colPoints As Collection
Dim sTmpCoords As String
Dim tmpX As Long, tmpY As Long

Set colPoints = New Collection

'\\ If coordinates are not set then default them
If mCoords = "" Then
    If Shape = imsCircle Then
        mCoords = "10,10,15,15"
    ElseIf Shape = imsRectangle Then
        mCoords = "10,10,20,20"
    Else
        mCoords = "5,5,10,10,15,15"
    End If
End If

sTmpCoords = mCoords

While sTmpCoords <> ""
  '\\ mCoords is in x,y pairs
  If InStr(sTmpCoords, ",") > 0 Then
    tmpX = Val(sTmpCoords)
    sTmpCoords = Mid$(sTmpCoords, InStr(sTmpCoords, ",") + 1)
  End If
  If InStr(sTmpCoords, ",") > 0 Then
    tmpY = Val(sTmpCoords)
    sTmpCoords = Mid$(sTmpCoords, InStr(sTmpCoords, ",") + 1)
  ElseIf sTmpCoords > "" Then
    tmpY = Val(sTmpCoords)
    sTmpCoords = ""
  End If
  '\\ Save these details as a new ApiPoint object
  Set ptThis = New APIPoint
  ptThis.x = tmpX
  ptThis.y = tmpY
  '\\ Add this point to the collection
  colPoints.Add ptThis
Wend

Set CoordinatePoints = colPoints

End Property



Friend Property Let CoordString(ByVal newString As String)

mCoords = newString

End Property

Friend Property Get CoordString() As String
    
    CoordString = mCoords
    
End Property


Public Property Let HTMLText(ByVal newText As String)

    If newText <> Me.HTMLText Then
        msId = ImageMapSubs.GetValue(newText, HTML_ID)
        Select Case GetValue(newText, HTML_SHAPE)
        Case "rect", "rectangle"
            mShape = imsRectangle
        Case "circ", "circle"
            mShape = imsCircle
        Case "poly", "polygon"
            mShape = imsPolygon
        End Select
        mCoords = GetValue(newText, HTML_COORDINATES)
        mHref = GetValue(newText, HTML_HREF)
        Select Case GetValue(newText, HTML_TARGET)
        Case "_blank"
            msTarget = imt_blank
        Case "_parent"
            msTarget = imt_parent
        Case "_search"
            msTarget = imt_search
        Case "_self"
            msTarget = imt_self
        Case "_top"
            msTarget = imt_top
        Case Else
            msTarget = imt_none
        End Select
    End If
    
End Property

'\\ --[HTMLText]---------------------------------------------------------------
'\\ Returns this ImageMapItem as it would appear in an HTML document as an
'\\ <area > tag.
'\\ Format:
'\\ <area shape="rectangle"
'\\      target = "_top"
'\\      coords = "1,1,2,2"
'\\      href = "http://www.merrioncomputing.com/Download/index.htm"
'\\      id="toprect">
'\\ ---------------------------------------------------------------------------
Public Property Get HTMLText() As String

Dim sArea As String

sArea = HTML_AREATAG & " " & HTML_AREATAGCLOSE

'\\ Write the id
Call SetValue(HTML_ID, msId, sArea)

'\\ Write the shape= section
Select Case mShape
Case imsCircle
    Call SetValue(HTML_SHAPE, "circle", sArea)
Case imsRectangle
    Call SetValue(HTML_SHAPE, "rectangle", sArea)
Case imsPolygon
    Call SetValue(HTML_SHAPE, "polygon", sArea)
End Select

'\\ Coordinates
Call SetValue(HTML_COORDINATES, mCoords, sArea)


Call SetValue(HTML_HREF, mHref, sArea)

HTMLText = sArea

End Property

Public Property Let ImageMapItemId(ByVal newid As String)

    msId = newid
    
End Property

Public Property Get ImageMapItemId() As String
    
    ImageMapItemId = msId
    
End Property

Friend Sub Refresh()

mHrgn = 0

End Sub

'\\ --[RegionHandle]---------------------------------------------------------
'\\ Returns the region handle (if any) that
'\\ -------------------------------------------------------------------------
Friend Property Get RegionHandle() As Long

Dim ptThis As APIPoint
Dim arPoints() As POINTAPI
Dim lPoint As Long

If mHrgn = 0 Then
    If mCoords = "" Then
        '\\ No region can sensibly be created
        mHrgn = 0
    Else
        Select Case Me.Shape
        Case imsCircle
            '\\ Create a circular region from (x1,y1)(x2,y2)
            '\\ where x1,y1 is the circle centre and x2,y2 is
            '\\ the radius point
            With CoordinatePoints
                mHrgn = CreateEllipticRgnApi((.item(1).x - .item(2).x), _
                                         (.item(1).y - .item(2).y), _
                                         (.item(1).x + .item(2).x), _
                                         (.item(1).y + .item(2).y))
            End With
            
            
        Case imsRectangle
            '\\ Creates a rectangle region from (x1,y1)(x2,y2) where
            '\\ x1,y1 is the top left corner and x2,y2 is the bottom right
            With CoordinatePoints
                mHrgn = CreateRectRgnApi(.item(1).x, .item(1).y, .item(2).x, .item(2).y)
            End With
            
        Case imsPolygon
            '\\ Create a polygon region from (x1,y1)(x2,y2)..(xn,yn)
            '\\ being the series of points that outline the region
            For Each ptThis In CoordinatePoints
                ReDim Preserve arPoints(lPoint) As POINTAPI
                With ptThis
                    arPoints(lPoint).x = .x
                    arPoints(lPoint).y = .y
                End With
                lPoint = lPoint + 1
            Next ptThis
            If lPoint > 0 Then
                mHrgn = CreatePolygonRgnApi(arPoints(0), lPoint, 1)
            End If
        End Select
    End If
End If
RegionHandle = mHrgn
    
End Property

Public Property Let Shape(ByVal newShape As enImageMapItemShapes)

    mShape = newShape
    
End Property

Public Property Get Shape() As enImageMapItemShapes

    Shape = mShape
    
End Property

Public Property Let Target(ByVal newTarget As enImageMapTargets)

    msTarget = newTarget
    
End Property

Public Property Get Target() As enImageMapTargets

    Target = msTarget
    
End Property


